@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2017	PgPool Global Development Group
+ * Copyright (c) 2003-2023	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -90,6 +90,12 @@ typedef struct
 									 * extended query, do not commit cache if
 									 * this flag is true. */
 
+	bool		atEnd;				/* if true all rows have been already
+									 * fetched from the portal */
+
+	bool		partial_fetch;		/* if true some rows have been fetched by
+									 * an execute with non 0 row option */
+
 	MemoryContext memory_context;	/* memory context for query context */
 }			POOL_QUERY_CONTEXT;
 
@@ -127,4 +133,6 @@ extern void pool_unset_cache_exceeded(void);
 extern bool pool_is_transaction_read_only(Node *node);
 extern void pool_force_query_node_to_backend(POOL_QUERY_CONTEXT * query_context, int backend_id);
 extern void check_object_relationship_list(char *name, bool is_func_name);
+extern int wait_for_failover_to_finish(void);
+
 #endif							/* POOL_QUERY_CONTEXT_H */
