@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2018	PgPool Global Development Group
+ * Copyright (c) 2003-2025	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -26,6 +26,7 @@
 #define WD_UTILS_H
 
 #include <ifaddrs.h>
+#include <pthread.h>
 #include "parser/pg_list.h"
 
 #define WD_NG (0)
@@ -52,6 +53,7 @@ extern int	wd_is_upper_ok(char *server_list);
 extern bool wd_is_ip_exists(char *ip);
 extern bool wd_get_ping_result(char *hostname, int exit_status, int outfd);
 extern pid_t wd_issue_ping_command(char *hostname, int *outfd);
+extern pid_t wd_trusted_server_command(char *hostname);
 
 /* wd_if.c */
 extern List *get_all_local_ips(void);
