@@ -3,7 +3,7 @@
  * pgpool_adm.c
  *
  *
- * Copyright (c) 2002-2019, PostgreSQL Global Development Group
+ * Copyright (c) 2002-2021, PostgreSQL Global Development Group
  *
  * Author: Jehan-Guillaume (ioguix) de Rorthais <jgdr@dalibo.com>
  *
@@ -38,7 +38,7 @@
 
 static PCPConnInfo * connect_to_server(char *host, int port, char *user, char *pass);
 static PCPConnInfo * connect_to_server_from_foreign_server(char *name);
-
+static Timestamp	str2timestamp(char *str);
 
 /**
  * Wrapper around pcp_connect
@@ -130,12 +130,13 @@ _pcp_node_info(PG_FUNCTION_ARGS)
 	PCPResultInfo *pcpResInfo;
 
 	BackendInfo *backend_info = NULL;
-	Datum		values[9];		/* values to build the returned tuple from */
-	bool		nulls[] = {false, false, false, false, false, false, false, false, false};
+	Datum		values[11];		/* values to build the returned tuple from */
+	bool		nulls[] = {false, false, false, false, false, false, false, false, false, false, false};
 	TupleDesc	tupledesc;
 	HeapTuple	tuple;
 	struct tm	tm;
 	char		datebuf[20];
+	int			i;
 
 	if (nodeID < 0 || nodeID >= MAX_NUM_BACKENDS)
 		ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE), errmsg("NodeID is out of range.")));
@@ -176,63 +177,84 @@ _pcp_node_info(PG_FUNCTION_ARGS)
 	 * Construct a tuple descriptor for the result rows.
 	 **/
 #if defined(PG_VERSION_NUM) && (PG_VERSION_NUM >= 120000)
-	tupledesc = CreateTemplateTupleDesc(9);
+	tupledesc = CreateTemplateTupleDesc(11);
 #else
-	tupledesc = CreateTemplateTupleDesc(9, false);
+	tupledesc = CreateTemplateTupleDesc(11, false);
 #endif
 	TupleDescInitEntry(tupledesc, (AttrNumber) 1, "hostname", TEXTOID, -1, 0);
 	TupleDescInitEntry(tupledesc, (AttrNumber) 2, "port", INT4OID, -1, 0);
 	TupleDescInitEntry(tupledesc, (AttrNumber) 3, "status", TEXTOID, -1, 0);
-	TupleDescInitEntry(tupledesc, (AttrNumber) 4, "weight", FLOAT4OID, -1, 0);
-	TupleDescInitEntry(tupledesc, (AttrNumber) 5, "role", TEXTOID, -1, 0);
-	TupleDescInitEntry(tupledesc, (AttrNumber) 6, "replication_delay", INT8OID, -1, 0);
-	TupleDescInitEntry(tupledesc, (AttrNumber) 7, "replcation_state", TEXTOID, -1, 0);
-	TupleDescInitEntry(tupledesc, (AttrNumber) 8, "replcation_sync_state", TEXTOID, -1, 0);
-	TupleDescInitEntry(tupledesc, (AttrNumber) 9, "last_status_change", TIMESTAMPOID, -1, 0);
+	TupleDescInitEntry(tupledesc, (AttrNumber) 4, "pg_status", TEXTOID, -1, 0);
+	TupleDescInitEntry(tupledesc, (AttrNumber) 5, "weight", FLOAT4OID, -1, 0);
+	TupleDescInitEntry(tupledesc, (AttrNumber) 6, "role", TEXTOID, -1, 0);
+	TupleDescInitEntry(tupledesc, (AttrNumber) 7, "pg_role", TEXTOID, -1, 0);
+	TupleDescInitEntry(tupledesc, (AttrNumber) 8, "replication_delay", INT8OID, -1, 0);
+	TupleDescInitEntry(tupledesc, (AttrNumber) 9, "replication_state", TEXTOID, -1, 0);
+	TupleDescInitEntry(tupledesc, (AttrNumber) 10, "replication_sync_state", TEXTOID, -1, 0);
+	TupleDescInitEntry(tupledesc, (AttrNumber) 11, "last_status_change", TIMESTAMPOID, -1, 0);
 	tupledesc = BlessTupleDesc(tupledesc);
 
 	backend_info = (BackendInfo *) pcp_get_binary_data(pcpResInfo, 0);
 
 	/* set values */
-	values[0] = CStringGetTextDatum(backend_info->backend_hostname);
-	nulls[0] = false;
-	values[1] = Int16GetDatum(backend_info->backend_port);
-	nulls[1] = false;
+	i = 0;
+	values[i] = CStringGetTextDatum(backend_info->backend_hostname);
+	nulls[i] = false;
+	i++;
+	values[i] = Int16GetDatum(backend_info->backend_port);
+	nulls[i] = false;
+
+	i++;
 	switch (backend_info->backend_status)
 	{
 		case CON_UNUSED:
-			values[2] = CStringGetTextDatum("Connection unused");
+			values[i] = CStringGetTextDatum("Connection unused");
 			break;
 		case CON_CONNECT_WAIT:
-			values[2] = CStringGetTextDatum("Waiting for connection to start");
+			values[i] = CStringGetTextDatum("Waiting for connection to start");
 			break;
 		case CON_UP:
-			values[2] = CStringGetTextDatum("Connection in use");
+			values[i] = CStringGetTextDatum("Connection in use");
 			break;
 		case CON_DOWN:
-			values[2] = CStringGetTextDatum("Disconnected");
+			values[i] = CStringGetTextDatum("Disconnected");
 			break;
 	}
-	nulls[2] = false;
-	values[3] = Float8GetDatum(backend_info->backend_weight / RAND_MAX);
-	nulls[3] = false;
+	nulls[i] = false;
 
-	nulls[4] = false;
-	values[4] = backend_info->role == ROLE_PRIMARY ? CStringGetTextDatum("Primary") : CStringGetTextDatum("Standby");
+	i++;
+	nulls[i] = false;
+	values[i] = CStringGetTextDatum(backend_info->pg_backend_status);
 
-	nulls[5] = false;
-	values[5] = Int64GetDatum(backend_info->standby_delay);
+	i++;
+	values[i] = Float4GetDatum(backend_info->backend_weight / RAND_MAX);
+	nulls[i] = false;
 
-	nulls[6] = false;
-	values[6] = CStringGetTextDatum(backend_info->replication_state);
+	i++;
+	nulls[i] = false;
+	values[i] = backend_info->role == ROLE_PRIMARY ? CStringGetTextDatum("Primary") : CStringGetTextDatum("Standby");
 
-	nulls[7] = false;
-	values[7] = CStringGetTextDatum(backend_info->replication_sync_state);
+	i++;
+	nulls[i] = false;
+	values[i] = CStringGetTextDatum(backend_info->pg_role);
 
-	nulls[8] = false;
+	i++;
+	nulls[i] = false;
+	values[i] = Int64GetDatum(backend_info->standby_delay);
+
+	i++;
+	nulls[i] = false;
+	values[i] = CStringGetTextDatum(backend_info->replication_state);
+
+	i++;
+	nulls[i] = false;
+	values[i] = CStringGetTextDatum(backend_info->replication_sync_state);
+
+	i++;
+	nulls[i] = false;
 	localtime_r(&backend_info->status_changed_time, &tm);
 	strftime(datebuf, sizeof(datebuf), "%F %T", &tm);
-	values[8] = DatumGetTimestamp(DirectFunctionCall3(timestamp_in,
+	values[i] = DatumGetTimestamp(DirectFunctionCall3(timestamp_in,
 													  CStringGetDatum(datebuf),
 													  ObjectIdGetDatum(InvalidOid),
 													  Int32GetDatum(-1)));
@@ -571,3 +593,161 @@ _pcp_detach_node(PG_FUNCTION_ARGS)
 	PG_RETURN_BOOL(true);
 }
 
+/**
+ * nodeID: the node id to get info from
+ * host_or_srv: server name or ip address of the pgpool server
+ * port: pcp port number
+ * user: user to connect with
+ * pass: password
+ **/
+Datum
+_pcp_health_check_stats(PG_FUNCTION_ARGS)
+{
+	int16		nodeID = PG_GETARG_INT16(0);
+	char	   *host_or_srv = text_to_cstring(PG_GETARG_TEXT_PP(1));
+
+	PCPConnInfo *pcpConnInfo;
+	PCPResultInfo *pcpResInfo;
+
+	POOL_HEALTH_CHECK_STATS *stats;
+	Datum		values[20];		/* values to build the returned tuple from */
+	bool		nulls[] = {false, false, false, false, false, false, false, false, false, false,
+						   false, false, false, false, false, false, false, false, false, false};
+	TupleDesc	tupledesc;
+	HeapTuple	tuple;
+	AttrNumber	an;
+	int			i;
+
+	if (nodeID < 0 || nodeID >= MAX_NUM_BACKENDS)
+		ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE), errmsg("NodeID is out of range.")));
+
+	if (PG_NARGS() == 5)
+	{
+		char	   *user,
+				   *pass;
+		int			port;
+
+		port = PG_GETARG_INT16(2);
+		user = text_to_cstring(PG_GETARG_TEXT_PP(3));
+		pass = text_to_cstring(PG_GETARG_TEXT_PP(4));
+		pcpConnInfo = connect_to_server(host_or_srv, port, user, pass);
+	}
+	else if (PG_NARGS() == 2)
+	{
+		pcpConnInfo = connect_to_server_from_foreign_server(host_or_srv);
+	}
+	else
+	{
+		ereport(ERROR, (errcode(ERRCODE_INTERNAL_ERROR), errmsg("Wrong number of argument.")));
+	}
+
+	pcpResInfo = pcp_health_check_stats(pcpConnInfo, nodeID);
+	if (pcpResInfo == NULL || PCPResultStatus(pcpResInfo) != PCP_RES_COMMAND_OK)
+	{
+		char	   *error = pcp_get_last_error(pcpConnInfo) ? pstrdup(pcp_get_last_error(pcpConnInfo)) : NULL;
+
+		pcp_disconnect(pcpConnInfo);
+		pcp_free_connection(pcpConnInfo);
+		ereport(ERROR, (errcode(ERRCODE_INTERNAL_ERROR),
+						errmsg("failed to get node information"),
+						errdetail("%s\n", error ? error : "unknown reason")));
+	}
+
+	/**
+	 * Construct a tuple descriptor for the result rows.
+	 **/
+#if defined(PG_VERSION_NUM) && (PG_VERSION_NUM >= 120000)
+	tupledesc = CreateTemplateTupleDesc(20);
+#else
+	tupledesc = CreateTemplateTupleDesc(20, false);
+#endif
+	an = 1;
+	TupleDescInitEntry(tupledesc, an++, "node_id", INT4OID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "hostname", TEXTOID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "port", INT4OID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "status", TEXTOID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "role", TEXTOID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "last_status_change", TIMESTAMPOID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "total_count", INT8OID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "success_count", INT8OID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "fail_count", INT8OID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "skip_count", INT8OID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "retry_count", INT8OID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "average_retry_count", FLOAT4OID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "max_retry_count", INT8OID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "max_health_check_duration", INT8OID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "min_health_check_duration", INT8OID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "average_health_check_duration", FLOAT4OID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "last_health_check", TIMESTAMPOID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "last_successful_health_check", TIMESTAMPOID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "last_skip_health_check", TIMESTAMPOID, -1, 0);
+	TupleDescInitEntry(tupledesc, an++, "last_failed_health_check", TIMESTAMPOID, -1, 0);
+
+	tupledesc = BlessTupleDesc(tupledesc);
+
+	stats = (POOL_HEALTH_CHECK_STATS *) pcp_get_binary_data(pcpResInfo, 0);
+
+	/* set values */
+	i = 0;
+	values[i++] = Int32GetDatum(nodeID);
+	values[i++] = CStringGetTextDatum(stats->hostname);
+	values[i++] = Int32GetDatum(atoi(stats->port));
+	values[i++] = CStringGetTextDatum(stats->status);
+	values[i++] = CStringGetTextDatum(stats->role);
+
+	if (*stats->last_status_change == '\0')
+		nulls[i++] = true;
+	else
+		values[i++] = str2timestamp(stats->last_status_change);
+
+	values[i++] = Int64GetDatum(atol(stats->total_count));
+	values[i++] = Int64GetDatum(atol(stats->success_count));
+	values[i++] = Int64GetDatum(atol(stats->fail_count));
+	values[i++] = Int64GetDatum(atol(stats->skip_count));
+	values[i++] = Int64GetDatum(atol(stats->retry_count));
+	values[i++] = Float4GetDatum(atof(stats->average_retry_count));
+	values[i++] = Int64GetDatum(atol(stats->max_retry_count));
+	values[i++] = Int64GetDatum(atol(stats->max_health_check_duration));
+	values[i++] = Int64GetDatum(atol(stats->min_health_check_duration));
+	values[i++] = Float4GetDatum(atof(stats->average_health_check_duration));
+
+	if (*stats->last_health_check =='\0' )
+		nulls[i++] = true;
+	else
+		values[i++] = str2timestamp(stats->last_health_check);
+
+	if (*stats->last_successful_health_check == '\0')
+		nulls[i++] = true;
+	else
+		values[i++] = str2timestamp(stats->last_successful_health_check);
+
+	if (*stats->last_skip_health_check == '\0')
+		nulls[i++] = true;
+	else
+		values[i++] = str2timestamp(stats->last_skip_health_check);
+
+	if (*stats->last_failed_health_check == '\0')
+		nulls[i++] = true;
+	else
+		values[i++] = str2timestamp(stats->last_failed_health_check);
+
+	pcp_disconnect(pcpConnInfo);
+	pcp_free_connection(pcpConnInfo);
+
+	/* build and return the tuple */
+	tuple = heap_form_tuple(tupledesc, values, nulls);
+
+	ReleaseTupleDesc(tupledesc);
+
+	PG_RETURN_DATUM(HeapTupleGetDatum(tuple));
+}
+
+static
+Timestamp str2timestamp(char *str)
+{
+	return (DatumGetTimestamp(DirectFunctionCall3(timestamp_in,
+												  CStringGetDatum(str),
+												  ObjectIdGetDatum(InvalidOid),
+												  Int32GetDatum(-1))));
+}
+
