@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2023	PgPool Global Development Group
+ * Copyright (c) 2003-2025	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -48,7 +48,7 @@ static char *relation_cache_to_query_cache(POOL_SELECT_RESULT *res,size_t *size)
  */
 POOL_RELCACHE *
 pool_create_relcache(int cachesize, char *sql,
-					 func_ptr register_func, func_ptr unregister_func,
+					 void *(*register_func) (POOL_SELECT_RESULT *), void *(*unregister_func) (void *),
 					 bool issessionlocal)
 {
 	POOL_RELCACHE *p;
@@ -187,7 +187,7 @@ pool_search_relcache(POOL_RELCACHE * relcache, POOL_CONNECTION_POOL * backend, c
 				{
 					ereport(DEBUG1,
 							(errmsg("searching relcache"),
-							 errdetail("relcache for database:%s table:%s expired. now:%ld expiration time:%ld", dbname, table, now, relcache->cache[i].expire)));
+							 errdetail("relcache for database:%s table:%s expired. now:%lld expiration time:%lld", dbname, table, (long long)now, (long long)relcache->cache[i].expire)));
 
 					relcache->cache[i].refcnt = 0;
 					break;
